// Gem.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "tnt.h"
#include <MersenneTwister.h>

#include "MIL.h"

#include "winsock2.h"
typedef int socklen_t;				// unix type not yet in windows include ? 

#include "SimComm.h"

const int IDM_FIRSTCHILD=1000;
const char * DEFAULT_WORKSPACE = "Workspace/Default";

//-- global drag parameters...
int g_iXDrag;
int g_iYDrag;
bool g_bDrag=false;

//-- Load Menues 
HMENU g_hMenuInit;				// initial menu
HMENU g_hMenuData;				// data view popup menu
HMENU g_hMenuModel;				// model view popup menu

CMRUList g_oMRUWorkspace;			// Containers for Most Recently Used file lists.
CMRUList g_oMRUModel;
CMRUList g_oMRUData;
CMRUList g_oMRUMovie;

//-- Forward declarations
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK MainWndProc(HWND, UINT, WPARAM, LPARAM);

//-- special window - MDI frame client 
HWND g_hClientWin;				
HWND GetClientWin( ) { 
	return g_hClientWin;
	}	
		
//-- strings not in string table
const char * g_szFrameClass = "GEMMainWindow" ;
const char * g_szChildClass = "GEMDialogWindow" ;
const char * g_szGLClass = "GEMOpenGLWindow" ;
const char * g_szEmptyClass = "GEMEmptyWindow" ;
const char * g_szTitle = "GEM";
const LPCTSTR hIcon = (LPCTSTR) IDI_GEM;
const char * GetChildClassName() {
	return g_szChildClass;
	}

//----- WinMain app entry point and main message pump
int APIENTRY _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow ) {
 	//----- Set globals
	SetInstance( hInstance );
	RegistrySetPath( "Software\\GEM" );
 	
 	//----- Register Window Classes 
	CreateWindowClass( g_szFrameClass, (WNDPROC)MainWndProc, hInstance, hIcon, (LPCTSTR)IDI_SMALL, 0, COLOR_APPWORKSPACE );
	CreateWindowClass( g_szChildClass, (WNDPROC)ChildWndProc, hInstance, hIcon, (LPCTSTR)IDI_SMALL, sizeof(HANDLE), COLOR_BTNFACE /*COLOR_APPWORKSPACE*/ );
	CreateWindowClass( g_szGLClass, (WNDPROC)EnergyWndProc, hInstance, hIcon, (LPCTSTR)IDI_SMALL, sizeof(HANDLE), COLOR_BTNFACE );
	CreateWindowClass( g_szEmptyClass, (WNDPROC)HistogramWndProc, hInstance, hIcon, (LPCTSTR)IDI_SMALL, sizeof(HANDLE), COLOR_BTNFACE );
	RegisterReportClass( );
	
	//----- Menues load 
	//TODO - menues not owned by a window should be removed with DestroyMenu at exit, 
	// but it might not be required since windows will destroy all memory and handles.
	g_hMenuInit = LoadMenu( hInstance, (LPCTSTR)IDC_GEM );
	g_hMenuData = LoadMenu( hInstance, (LPCTSTR)IDR_MENUDATA );	
	g_hMenuModel = LoadMenu( hInstance, (LPCTSTR)IDR_MENUMODEL );
	
	//----- create and display the main program window.
	HACCEL hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_GEM);
	HWND hFrameWin = CreateWindow( g_szFrameClass, g_szTitle, 
							WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN, 
							CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, 
							NULL, g_hMenuInit,  hInstance, NULL);
	if( ! hFrameWin )
		return FALSE;
								
	g_hClientWin = GetWindow( hFrameWin, GW_CHILD );
	ShowWindow( hFrameWin, SW_MAXIMIZE );
	UpdateWindow( hFrameWin );

	//------ Required for common dialogs, etc.
	InitCommonControls();

	init_genrand( int(Now()) );

	CRptWin::WinInit( GetInstance(), false );
	
	#ifdef _DEBUG
		//---------------------------------------------------------------
		// This is a good place to put whatever is currently being tested 
		//--------------------------------------------------------------- 
	#endif


	HMENU mnuX, mnuFile = GetSubMenu( g_hMenuInit, 0 );

	mnuX = GetSubMenu( mnuFile, 1 );
	g_oMRUModel.Set( "MRUModel", IDM_MRUMODEL, mnuX, 5);
	g_oMRUModel.LoadList();
	g_oMRUModel.UpdateMenu();
			
	mnuX = GetSubMenu( mnuFile, 2 );
	g_oMRUData.Set( "MRUData", IDM_MRUDATA, mnuX, 5);
	g_oMRUData.LoadList();
	g_oMRUData.UpdateMenu();

	mnuX = GetSubMenu( mnuFile, 3 );
	g_oMRUMovie.Set( "MRUMovie", IDM_MRUMOVIE, mnuX, 5);
	g_oMRUMovie.LoadList();
	g_oMRUMovie.UpdateMenu();

	DrawMenuBar( GetParent(g_hClientWin) );

	//----- Main message loop:
	MSG msg;
	while( GetMessage(&msg, NULL, 0, 0) ) {
		if( ! WindowListProcessMessage( &msg ) 
			&& ! TranslateMDISysAccel( g_hClientWin, &msg ) 
			&& ! TranslateAccelerator( hFrameWin, hAccelTable, &msg)
			) {	//msg.hwnd
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			}
		}

	g_oMRUModel.SaveList();
	g_oMRUData.SaveList();
	g_oMRUMovie.SaveList();
	
	return (int) msg.wParam;
	}


//-----------------------------------------------------------------------------	
//  MainWndProc() -  Processes messages for the main window.
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam){
	int wmId, wmEvent;
	HWND hChildWin; //, hDlg; 

	switch (message) {
		case WM_CREATE : {
			//---- Create the client of the frame - an MDI oddity 
			CLIENTCREATESTRUCT ccs;
			ccs.hWindowMenu=GetSubMenu( GetMenu(hWnd), 0/*position of menu*/ );
			ccs.idFirstChild=IDM_FIRSTCHILD;
			g_hClientWin = CreateWindow( "MDICLIENT", NULL, 
								WS_CHILD|WS_CLIPCHILDREN| WS_VISIBLE, 
								0, 0, 0, 0, hWnd, 
								(HMENU) 0xCAC, //  GetSubMenu(GetMenu(hwnd), WINDOWMENU), 
								GetInstance(), &ccs );
			
			//---- create the report window 
			//CreateReportWindow(); - if initial display of error window is desired
			AddMessage( REPTYPE_NOTE, "GEM STARTED" );
			CreateReportWindow( g_hClientWin );

			//RECT rct;
			//GetClientRect( g_hClientWin, &rct );
			//MoveMessageWindow( 10, max(rct.bottom-200,10), rct.right-20, 200 );
			MoveMessageWindow( 10, 600, 780, 200 );
			
			//-- open default workspace - all previously opened windows
 			LoadWorkspace( RegistryLoadString( DEFAULT_WORKSPACE ).ConstBuffer() );
			}			
			return 0;

		case WM_COMMAND:
			//----- Parse the menu selections:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			switch (wmId) {
				//----- workspace open / close 
				case ID_WORKSPACE_OPEN : {
					char achLongName[_MAX_PATH]={0};
					char achFileName[_MAX_PATH]={0};
					CString c;
					if( GetOpenFileName( achLongName, "Gem Workspace(*.gwk)\0*.gwk\0\0", "gwk", hWnd, achFileName ) 
							&& StringFromFile( achLongName, &c ) ) 
			 			LoadWorkspace( c.ConstBuffer() );
					}
					break;
					
				case ID_WORKSPACE_SAVE : {
					char achLongName[_MAX_PATH]="workspace.gwk";
					char achFileName[_MAX_PATH]="workspace.gwk";
					if( GetSaveFileName( achLongName, "Gem Workspace(*.gwk)\0*.gwk\0\0", "gwk", hWnd, achFileName ) ) {
						CString c = SaveWorkspace();
						StringToFile( &c, achLongName ); 						
						}
					}
					break;
			
				//----- Open the About Dialogue - Modal 
				case IDM_ABOUT:
					DialogBox( GetInstance(), (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
					break;

				//----- Display the error window 
				case ID_VIEW_ERRORWINDOW : 
					CreateReportWindow( g_hClientWin );
					break;
				
				//----- Display the graphs window 
				case ID_FILE_GRAPHS : 
					if( WindowListFindID( IDD_GRAPHS )==-1 ) 
						OpenGraphsView( "Graphs" );
					else 
						OutputDebugString("Already open\n");
					break;
				
				//----- add a histogram to the graphs window 
				case WM_ADD_HISTOGRAM : { 
					CHistogram * pHist = (CHistogram *)lParam;
					pHist->SetXScale();
					pHist->SetYScale();

					if( WindowListFindID( IDD_GRAPHS )==-1 ) 
						OpenGraphsView( "Graphs" );
					AddGraph( pHist );
					}
					break;
								
				//----- Open Model from Most Recently Used list
				case IDM_MRUMODEL+0 : 
				case IDM_MRUMODEL+1 : 
				case IDM_MRUMODEL+2 : 
				case IDM_MRUMODEL+3 : 
				case IDM_MRUMODEL+4 :
					if( wmId-IDM_MRUMODEL<g_oMRUModel.m_aItems.Count() ) {
						char * pchName = ((CMRUItem *) g_oMRUModel.m_aItems[g_oMRUModel.m_aItems.Count()-1-(wmId-IDM_MRUMODEL)] )->m_cName.ConstBuffer(); 
						char achLongName[_MAX_PATH];
						char achFileName[_MAX_PATH];
						strcpy(achLongName,pchName);
						strcpy(achFileName,pchName);
						g_oMRUModel.AddItem( achLongName );
						g_oMRUModel.UpdateMenu();
						SendMessage( OpenModelView( achFileName ), WM_MODEL_LOADFILE, 0, (LPARAM)achLongName );
						}
					break;
				
				//----- File->Model->New		Open the model view with a new basic model ( closed - open ) 
				case ID_MODEL_NEW :
					SendMessage( OpenModelView( "New Model" ), WM_MODEL_NEWFILE, 0, 0 );
					break;

				//----- File->Model->Open		Open the model edit window 
				case ID_MODELFILE_OPEN : {
					//-- select the file to load 
					char achLongName[_MAX_PATH]={0};
					char achFileName[_MAX_PATH]={0};
					if( ! GetOpenFileName( achLongName, 
									"Model files (*.qmf)\0*.qmf\0All Files (*.*)\0*.*\0\0", 
									"qmf", GetParent(hWnd), achFileName ) ) 
						return 0;
									  
					//-- Open the Model window 
					g_oMRUModel.AddItem( achLongName );
					g_oMRUModel.UpdateMenu();
					
					SendMessage( OpenModelView( achFileName), WM_MODEL_LOADFILE, 0, (LPARAM)achLongName );
					}
					break;
				
				//----- Open the energy model window 
				/*
				case ID_ENERGYMODEL_OPEN : {
					//-- select the file to load 
					char achLongName[_MAX_PATH]={0};
					char achFileName[_MAX_PATH]={0};
					if( ! GetOpenFileName( achLongName, 
									"Model files (*.qmf)\0*.qmf\0All Files (*.*)\0*.*\0\0", 
									"qmf", GetParent(hWnd), achFileName ) ) 
						return 0;
					//-- Open the energy landscape window 
					hChildWin = CreateMDIWindow( g_szGLClass, "Energy Landscape", NULL, 
									CW_USEDEFAULT, CW_USEDEFAULT, 716, 620, 
									g_hClientWin, GetInstance(), 0 );
					SendMessage( hChildWin, WM_MODEL_OPEN, 0, (LPARAM)achLongName );
					}
					break;
				*/
					
				//----- Open the energy model window 
				case ID_ENERGYMODEL_VIEW : {
					//-- Open the energy landscape window 
					hChildWin = CreateMDIWindow( g_szGLClass, "Energy Landscape", NULL, 
									CW_USEDEFAULT, CW_USEDEFAULT, 716, 620, 
									g_hClientWin, GetInstance(), 0 );
					SendMessage( hChildWin, WM_MODEL_OPEN, 1, lParam /* &m_oModel */);
					}
					break;

				//----- Open Data from Most Recently Used list
				case IDM_MRUDATA+0 : 
				case IDM_MRUDATA+1 : 
				case IDM_MRUDATA+2 : 
				case IDM_MRUDATA+3 : 
				case IDM_MRUDATA+4 :
					if( wmId-IDM_MRUDATA<g_oMRUData.m_aItems.Count() ) {
						char * pchName = ((CMRUItem *) g_oMRUData.m_aItems[g_oMRUData.m_aItems.Count()-1-(wmId-IDM_MRUDATA)])->m_cName.ConstBuffer(); 
						char achLongName[_MAX_PATH];
						char achFileName[_MAX_PATH];
						strcpy(achLongName,pchName);
						strcpy(achFileName,pchName);
						g_oMRUData.AddItem( achLongName );
						g_oMRUData.UpdateMenu();
						SendMessage( OpenDataView(achFileName), WM_DATA_LOADFILE, 0, (LPARAM)achLongName );
						}
					break;
					
				//----- File->Data->Open		Open a data file 
				case ID_DATAFILE_OPEN : {
					//-- select the file to load 
					char achLongName[_MAX_PATH]={0};
					char achFileName[_MAX_PATH]={0};
					if( ! GetOpenFileName( achLongName, 
									"Data files (*.qdf)\0*.qdf\0All Files (*.*)\0*.*\0\0", 
									"qdf", GetParent(hWnd), achFileName ) ) 
						return 0;
									  
					//-- Open the data window 
					g_oMRUData.AddItem( achLongName );
					g_oMRUData.UpdateMenu();
					SendMessage( OpenDataView(achFileName), WM_DATA_LOADFILE, 0, (LPARAM)achLongName );
					}
					break;
				case ID_DATA_DEFAULTS : 
					OpenDataDefaultConfig( hWnd ); 
					break;

				//----- "Extras"
				//-- Model Generator
				case ID_EXTRAS_MODELGENERATOR : 
					OpenModelGenerator( hWnd );
					break;

				//-- Model Merge
				case ID_EXTRAS_MODELMERGE : 
					OpenVwModelMerge( hWnd );
					break;

				//----- Open Movie from Most Recently Used list
				case IDM_MRUMOVIE+0 : 
				case IDM_MRUMOVIE+1 : 
				case IDM_MRUMOVIE+2 : 
				case IDM_MRUMOVIE+3 : 
				case IDM_MRUMOVIE+4 : 
					if( wmId-IDM_MRUMOVIE<g_oMRUMovie.m_aItems.Count() ) {
						char * pchName = ((CMRUItem *) g_oMRUMovie.m_aItems[g_oMRUMovie.m_aItems.Count()-1-(wmId-IDM_MRUMOVIE)] )->m_cName.ConstBuffer(); 
						char achLongName[_MAX_PATH];
						char achFileName[_MAX_PATH];
						strcpy(achLongName,pchName);
						strcpy(achFileName,pchName);
						g_oMRUMovie.AddItem( achLongName );
						g_oMRUMovie.UpdateMenu();
						SendMessage( OpenMovieView(achFileName), WM_MOVIE_LOADFILE, 0, (LPARAM)achLongName );
						}
					break;
					
				//----- File->Movie->New
				case ID_MOVIE_NEW :
					SendMessage( OpenMovieView(NULL), WM_MOVIE_NEWFILE, 0, 0 );
					break;

				//----- File->Movie->Open
				case ID_MOVIE_OPEN : {
					//-- select the file to load 
					char achLongName[_MAX_PATH]={0};
					char achFileName[_MAX_PATH]={0};
					if( ! GetOpenFileName( achLongName, 
									"Movie files (*.gmv)\0*.gmv\0All Files (*.*)\0*.*\0\0", 
									"gmv", GetParent(hWnd), achFileName ) ) 
						return 0;

					g_oMRUMovie.AddItem( achLongName );
					g_oMRUMovie.UpdateMenu();
					SendMessage( OpenMovieView(NULL), WM_MOVIE_LOADFILE, 0, (LPARAM)achLongName );
					}
					break;

				//----- Exit ... 
				case IDM_EXIT:
		            SendMessage (hWnd, WM_CLOSE, 0, 0) ;
					break;
                
                case ID_WINDOW_CLOSE : {
					HWND h = (HWND) SendMessage (g_hClientWin, WM_MDIGETACTIVE, 0, 0) ;
					if( SendMessage( h, WM_QUERYENDSESSION, 0, 0) )
						SendMessage( g_hClientWin, WM_MDIDESTROY, (WPARAM) h, 0) ;
					}
					return 0 ;

				//---- Model Menu choices - pass to child window handler 
				case ID_MODEL_MIL : 
				case ID_MODEL_STATISTICS : 
				case ID_MODEL_SIMULATE : 
					ModelViewDlgProc( 
						GetWindow( (HWND) SendMessage (g_hClientWin, WM_MDIGETACTIVE, 0, 0), GW_CHILD ), 
						WM_COMMAND, wmId, 0);
					break;
								
				//---- Data Menu choices - pass to child window handler 
				case ID_DATACURSOR_NEW :
				case ID_DATACURSOR_DELETE :
				case ID_DATACURSOR_DELETEALL :
				case ID_DATALIST_NEW :
				case ID_DATALIST_DELETE :
				case ID_DATALIST_DELETEALL :
				case ID_DATALIST_NEWITEM :
				case ID_DATALIST_DELETEITEM :
				case ID_DATALIST_ADDALLSEGMENTS :
				case ID_DATAFITCURVE_NEW :
				case ID_DATAFITCURVE_DELETE :
				case ID_DATAFITCURVE_DELETEALL :
				case ID_DATAIDL_NEW :
				case ID_DATAIDL_DELETE :
				case ID_DATAIDL_DELETEALL :
				case ID_DATAANALYZE_MIL :
				case ID_DATAEXPORT_ASCII :
				case ID_DATAEXPORT_IDEALIZATION : 
				case ID_ANALYZE_AMPHISTOGRAM : 
					DataviewDlgProc( 
						GetWindow( (HWND) SendMessage (g_hClientWin, WM_MDIGETACTIVE, 0, 0), GW_CHILD ), 
						WM_COMMAND, wmId, 0);
					break;
					
				default : {            
					HWND h = (HWND) SendMessage( g_hClientWin, WM_MDIGETACTIVE, 0, 0 );
					if( IsWindow (h) )	// Pass to active child...
						SendMessage( h, WM_COMMAND, wParam, lParam );
					}
					break ;        // ...and then to DefFrameProc
				}
				break;

		case WM_CLOSE:
			#ifdef _DEBUG
			OutputDebugString("WM_CLOSE\n");
			#endif
			//-- check if all windows can close.		
			if( ! WindowListCanClose() )
				return 0;	// TODO - consider what to do here - option to force close ? 
				
			//-- check if user really wanted to close.
			if( MessageBox( NULL, "Exit program ?", "GEM", MB_YESNO ) != IDYES )
				return 0;
				
			//--- save workspace file list 
			RegistrySaveString( DEFAULT_WORKSPACE, SaveWorkspace().ConstBuffer() );

			//-- Tell all to close, which each may include a "Save?" query 
			WindowListClose();

			//-- all set, go on to default windows processing 				
			break;
			
		case WM_QUIT:
			#ifdef _DEBUG
			OutputDebugString("WM_QUIT\n");
			#endif
			break;
			
		case WM_DESTROY:
			#ifdef _DEBUG
			OutputDebugString("WM_DESTROY\n");
			#endif
			PostQuitMessage(0);
			DestroyMenu( g_hMenuData );
			DestroyMenu( g_hMenuModel );
			return 0;
		}
	
	LRESULT lr = DefFrameProc( hWnd, g_hClientWin, message, wParam, lParam);
	return lr;
	}

// Message handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam){
	switch (message){
		case WM_INITDIALOG:
			return TRUE;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
				}
			break;
		}
		
	return FALSE;
	}

//---- Switch menues for MDI clients 
void DataMenuOn() { 
	InsertMenu( g_hMenuInit, 2, MF_BYPOSITION|MF_POPUP, (UINT_PTR)g_hMenuData, "Data" ); 
	DrawMenuBar( GetParent(g_hClientWin) );
	}

void DataMenuOff() { 
	RemoveMenu( g_hMenuInit, 2, MF_BYPOSITION );
	DrawMenuBar( GetParent(g_hClientWin) );
	}

void ModelMenuOn() { 
	InsertMenu( g_hMenuInit, 2, MF_BYPOSITION|MF_POPUP, (UINT_PTR)g_hMenuModel, "Model" ); 
	DrawMenuBar( GetParent(g_hClientWin) );
	}
	
void ModelMenuOff() { 
	RemoveMenu( g_hMenuInit, 2, MF_BYPOSITION );
	DrawMenuBar( GetParent(g_hClientWin) );
	}

void MenuItemCheck( int iID, bool bCheck ){
	CheckMenuItem( g_hMenuInit, iID, MF_BYCOMMAND|(bCheck?MF_CHECKED:MF_UNCHECKED) );
	DrawMenuBar( GetParent(g_hClientWin) );
	}

void UpdateModelMRU( const char * pchName ) {
	g_oMRUModel.AddItem( pchName );
	g_oMRUModel.UpdateMenu();
	}

void UpdateDataMRU( const char * pchName ) {
	g_oMRUData.AddItem( pchName );
	g_oMRUData.UpdateMenu();
	}

void UpdateMovieMRU( const char * pchName ) {
	g_oMRUMovie.AddItem( pchName );
	g_oMRUMovie.UpdateMenu();
	}

//=============================================================================
void ThreadErrorMessage( const char * pchMsg, void * pInfo ) { 
	//TODO TODO Use WM_USER message to pass to front thread and process.
	OutputDebugString( pchMsg );
	}


//=============================================================================
HWND OpenDlgWindow( const char * pchWindowName, int iID, DLGPROC lpDialogFunc ) { 
	//-- Open the data window 
	HWND hChildWin = CreateMDIWindow( g_szChildClass, pchWindowName, NULL, 
							CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, 
							g_hClientWin, GetInstance(), 0 );
							
	HWND hDlg = CreateDialog( GetInstance(), MAKEINTRESOURCE(iID), hChildWin, lpDialogFunc );							
	WindowListAdd( hChildWin, hDlg );
	return hDlg;
	}

HWND OpenDataView( const char * pchWindowName ) { 
	return OpenDlgWindow( pchWindowName, IDD_DATAVIEW, DataviewDlgProc );
	}

HWND OpenModelView( const char * pchWindowName ) { 
	return OpenDlgWindow( pchWindowName, IDD_MODELVIEW, ModelViewDlgProc  );
	}

HWND OpenMovieView( const char * pchWindowName ) { 
	return OpenDlgWindow( pchWindowName, IDD_MOVIEVIEW, MovieViewDlgProc );
	}

HWND OpenGraphsView( const char * pchWindowName ) { 
	return OpenDlgWindow( pchWindowName, IDD_GRAPHS, GraphsViewDlgProc );
	}

HWND OpenHistogramView( const char * pchWindowName ) { 
	return CreateMDIWindow( g_szEmptyClass, pchWindowName, NULL, 
					CW_USEDEFAULT, CW_USEDEFAULT, 700, 400, 
					g_hClientWin, GetInstance(), 0 );
	}
	
//------------------------------ General operations
//-- Fill in combobox with list of open model names.  Return number of entries.
int FillModelCombo( CDlgCombo * pcmb, CStringArray * pac, CPtrArray * pahwnd ) { 
	if( pcmb != NULL ) 
		pcmb->ClearContents();

	//-- check if any open models
	int iWin = WindowListFindID( IDD_MODELVIEW, 0 );
	if( iWin == -1 )
		return 0;
	
	//-- iterate open models
	int iEntries=0;
	while( iWin != -1 ) { 
		HWND hwndModel = WindowListHandle( iWin );
		if( pahwnd != NULL ) 
			pahwnd->AddPtr( hwndModel );

		char * pchFile = (char *)(((CDialog *) GetWindowLong( GetParent(hwndModel), 0 ))->GetData( 1, NULL ));
		if( pac != NULL ) 
			pac->AddString( pchFile );
		if( pcmb != NULL ) 
			pcmb->AddString( pchFile );
			
		iWin = WindowListFindID( IDD_MODELVIEW, iWin+1 );
		++iEntries;
		}
		
	if( pcmb != NULL ) 
		pcmb->SetCurSel(0);
		
	return iEntries;
	}

void GetFileNameList( int iID, CStringArray * pac ) { 
	int iWin = WindowListFindID( iID, 0 );
	char * pchFile;
	HWND hwnd;
	while( iWin != -1 ) { 
		hwnd = WindowListHandle( iWin );
		pchFile = (char *)(((CDialog *) GetWindowLong( GetParent(hwnd), 0 ))->GetData( 1, NULL ));
		pac->AddString( pchFile );
		iWin = WindowListFindID( iID, iWin+1 );
		}
	}


//-- Save list of open models, data to registry
// (000)(3 digit file count)(3 digit string length 1)...(3 digit string length n)
//		(1 digit file type 1)...(1 digit file type 1)(string 1)...(string n)
//ex : "  3 12 16  8MDVC:\temp\fileC:\temp\file.txtC:\X.Dat"
CString SaveWorkspace( ) { 
	CStringArray acModels, acData, acMovies;
	GetFileNameList( IDD_MODELVIEW, &acModels );
	GetFileNameList( IDD_DATAVIEW, &acData );
	GetFileNameList( IDD_MOVIEVIEW, &acMovies );
	int iModels=acModels.Count();
	int iData=acData.Count();
	int iMovies=acMovies.Count();
	int iFile, iFiles=iModels+iData+iMovies;

	CString c("000");
	c += CString( iFiles, "%3d" );
	CString cTypes;
	CString cNames;
	for( iFile=0; iFile<iModels; ++iFile ) {
		c += CString( acModels[iFile].Length(), "%3d" );
		cTypes += "M";
		cNames += acModels[iFile];
		}
	for( iFile=0; iFile<iData; ++iFile ) {
		c += CString( acData[iFile].Length(), "%3d" );
		cTypes += "D";
		cNames += acData[iFile];
		}
	for( iFile=0; iFile<iMovies; ++iFile ) {
		c += CString( acData[iFile].Length(), "%3d" );
		cTypes += "V";
		cNames += acMovies[iFile];
		}

	c += cTypes;
	c += cNames;
	return c;
	}
	
//-- open list of models , data 
void LoadWorkspace( const char * pchKey ) { 
	CString cFile, c( pchKey );

	int iLen, iFile, iFiles=atoi(c.Mid(3,3).ConstBuffer());
	char chType;

	int iPos=iFiles*4+6;
	for( iFile=0; iFile<iFiles; ++iFile ) { 
		iLen = atoi(c.Mid(6+iFile*3,3).ConstBuffer() );
		chType = c[6+iFiles*3+iFile];
		cFile = c.Mid( iPos, iLen );
		
		if( FileExists(cFile.ConstBuffer()) ) { 
			if( chType == 'M' ) 		
				SendMessage( OpenModelView(cFile.ConstBuffer()), WM_MODEL_LOADFILE, 0, (LPARAM)cFile.ConstBuffer() );
			else if( chType == 'D' ) 		
				SendMessage( OpenDataView(cFile.ConstBuffer()), WM_DATA_LOADFILE, 0, (LPARAM)cFile.ConstBuffer() );
			else if( chType == 'V' ) 		
				SendMessage( OpenMovieView(cFile.ConstBuffer()), WM_MOVIE_LOADFILE, 0, (LPARAM)cFile.ConstBuffer() );
			}

		iPos += iLen;
		}
		
	}

